/*
	Zonic System V0.21 alpha
	
	1996 by Tinic Urou. Distribution of this
	sources is strictly prohibited.

 _________________________________________________________	
|	A simple button with a textlabel
|_________________________________________________________
*/
#include "zbutton.h"

// You must provide two Constructors. One varargs version and
// one pointer version. Like in this case, its sometimes
// useful to create an additional constructor method.

ZButton::ZButton(TAG_ARG):ZGroup(TAG_SUPER)			{ ZButtonConstructor(TAG_SRC); }
ZButton::ZButton(struct TagItem *tags):ZGroup(tags)	{ ZButtonConstructor(tags); }

void ZButton::ZButtonConstructor(struct TagItem *tags)
{
	char lab[128],i,d=0;
	zSetFrameType((char)GetTagData(tags,ZGroup_Frame_Button,ZGroup_Frame));
	zSetPen((char) GetTagData(tags,PEN_HALFSHINE,ZGroup_Background));
	strcpy(lab,(char *)GetTagData(tags,(ulong)"ZButton",ZButton_Label));	
	for(i=0,markPos=-1;i<=strlen(lab);i++)
	{
		if(lab[i]!='_')	label[d++]=lab[i];
		else			markPos=i;
	}
	msgup=(BMessage *)GetTagData(tags,NULL,ZButton_MessageUp);
	msgdown=(BMessage *)GetTagData(tags,NULL,ZButton_MessageDown);
	strcpy(fontname,(char *)GetTagData(tags,(ulong)GROUP_NORMAL_FONT,ZButton_FontName));
	fontsize=(float)GetTagData(tags,GROUP_NORMAL_HEIGHT,ZButton_FontSize);
	autoscale=(bool)GetTagData(tags,FALSE,ZButton_FontAutoScale);
	shortKey=(char)GetTagData(tags,NULL,ZButton_ShortCut);
}

// This is the Keydown hook if we are focused

void ZButton::zFocusKeyDown(ulong aKey)
{
	if(aKey==B_RETURN)
	{
		zSetState(ZGroup_State_Selected);
		zSetFrameState(ZGroup_FrameState_Selected);
		if(msgup)zPostMessage(new BMessage(msgup));
		zWindow()->Lock();
		zDraw(zFrame());
		zWindow()->Unlock();
		snooze(100000);
		zSetState(ZGroup_State_Normal);
		zSetFrameState(ZGroup_FrameState_Normal);
		zWindow()->Lock();
		zDraw(zFrame());
		zWindow()->Unlock();
	}
}

// General Keydown. Return yourself, if input 
// was accepted, else 0. It should be preferably 
// used for ShortCuts
// The priority of this hook is higher than of
// zFocusKeyDown()

ZGroup *ZButton::zKeyDown(ulong aKey)
{
	if(aKey==shortKey)
	{
		zSetState(ZGroup_State_Selected);
		zSetFrameState(ZGroup_FrameState_Selected);
		if(msgup)zPostMessage(new BMessage(msgup));
		zWindow()->Lock();
		zDraw(zFrame());
		zWindow()->Unlock();
		snooze(100000);
		zSetState(ZGroup_State_Normal);
		zSetFrameState(ZGroup_FrameState_Normal);
		zWindow()->Lock();
		zDraw(zFrame());
		zWindow()->Unlock();
		return this;
	}
	return 0;
}

// This hook gives you pointers to the ZView and ZWindow.
// Since these are not known on creation you should some of
// your ZView and ZWindow releated things here. Its like
// an AttachedToWindow().
// Its useful to get your StringWidths here, since this hook
// is only called once.
// NOTE: The special Zonic classes are not derived from
// the BView class. You must use either view or zView()
// to get access to the BView methods.

void ZButton::zRenderInterfaceSetup(ZView *view, ZWindow *window)
{	
	window->Lock();
	view->SetFontName(fontname);
	view->SetFontSize(fontsize);
	view->GetFontInfo(&finfo);
	labelwidth=view->StringWidth(label);
	
	underline=BRect(0,0,0,0);
	
	if(markPos>-1)
	{
		char c[2];c[0]=label[markPos];c[1]=0;
		underline.SetRightBottom(BPoint(view->StringWidth(c)-1,0));			
		underline.OffsetBy(view->StringWidth(label,markPos),finfo.ascent+5);
	}
	window->Unlock();
}

// This hook should contain the setting of the minimal size
// Dont forget to return the new size.

BPoint ZButton::zRenderPass1()
{
	return(zSetMinSize(BPoint(labelwidth+12,finfo.size+8.0)));
}

// This hook should contain the setting of the maximal size
// Dont forget to return the new size.

BPoint ZButton::zRenderPass2()
{
	return(zSetMaxSize(BPoint(GROUP_MAX_WIDTH,finfo.size+8.0)));
}

// This works equal to the Draw() hook of the BView
// If you use one of the standard framestypes and 
// backgroundpens which are provided by Zonic, call
// the zDraw() from the superclass.
// NOTE: The special Zonic classes are not derived from
// the BView class. You must use either view or zView()
// to get access to the BView methods. 
// Also take care, that most of the font and color settings
// in the BView wont stay at the same values. You should
// always reset all these settings in this hook.

void ZButton::zDraw(BRect rect)
{
	BPoint 	sel;
	BRect	und=underline;
	
	ZGroup::zDraw(rect);
	zView()->SetHighColor(TEXTCOLOR);
	zView()->SetDrawingMode(B_OP_OVER);
	zView()->SetFontName(fontname);
	zView()->SetFontSize(fontsize);

	if(zState()==ZGroup_State_Selected)	sel=BPoint(1,1);
	else 								sel=BPoint(0,0);

	zView()->DrawString(label,zFrame().LeftTop()+sel+BPoint((zFrame().Width()-labelwidth)/2,finfo.ascent+4));

	if(underline.right)
	{
		und.OffsetBy(zFrame().LeftTop()+sel+BPoint((zFrame().Width()-labelwidth)/2,0));
		zView()->FillRect(und);
	}
}

// e.g. MouseDown() from the BView class.

bool ZButton::zMouseDown(BPoint pos)
{
	ulong buttons;
	bool  selected=TRUE,oldsel=FALSE;
	
	if(zState()!=ZGroup_State_Disabled)
	{
		if(msgdown)zPostMessage(new BMessage(msgdown));
		do 
		{
			snooze(10000);
			if(!zFrame().Contains(pos))
			{
				selected=FALSE;
				if(oldsel==TRUE)
				{
					zSetState(ZGroup_State_Normal);
					zSetFrameState(ZGroup_FrameState_Normal);
					zWindow()->Lock();
					zDraw(zFrame());
					zWindow()->Unlock();
					oldsel=FALSE;
				}
			}
			else
			{
				selected=TRUE;
				if(oldsel==FALSE)
				{
					zSetState(ZGroup_State_Selected);
					zSetFrameState(ZGroup_FrameState_Selected);
					zWindow()->Lock();
					zDraw(zFrame());
					zWindow()->Unlock();
					oldsel=TRUE;
				}
			}
			zWindow()->Lock();
			zView()->GetMouse(&pos, &buttons);
			zWindow()->Unlock();
		}
		while(buttons);
		if(selected&&msgup)zPostMessage(new BMessage(msgup));
		zSetState(ZGroup_State_Normal);
		zSetFrameState(ZGroup_FrameState_Normal);
		zWindow()->Lock();
		zDraw(zFrame());
		zWindow()->Unlock();
	}
	return 1;
}
