/*
	zonicdemo V0.21 by Tinic Urou, 25. November 1996
	
	This little program shows the capabilities of
	the Zonic Layoutengine.
*/
#include "zpager.h"
#include "zbutton.h"
#include "zinterface.h"

// Two bitmaps in raw data format
extern unsigned char anImage[];
extern unsigned char BeImage[];

// Message IDs for upper buttons.
#define GROUP_1	'grp1'
#define GROUP_2	'grp2'
#define GROUP_3	'grp3'

ZGroup *g1,*g3,*mg,*k1,*k2,*k3,*k4;
ZBStringView *g2;
ZButton *b1,*b2,*b3;

// Message Handler for upper button Group
// We could also use one Message handler for the
// whole application, but I think, this method
// is much more flexible. Think f.ex. combined
// Group like a listview which is put together from
// ScrollBars and the real ListView

class Button_Group : public ZGroup
{
	public:

	// You have to provide these two Constructors!
	// They look always like this:
		
	Button_Group(TAG_ARG):ZGroup(TAG_SUPER) { }; 
	Button_Group(struct TagItem *tags):ZGroup(tags) { };
	
	// Dispatch Message we got
	// Note, that Zonic has an own Message System but 
	// uses BMessage objects.
	// External Message f.ex. drag&drop message are also
	// forwarded when the ZGroup_CanBeDropped is set and
	// zMessageFilter() is implemented.
	// Look to the next class for this...
	
	void zMessageReceived(BMessage *msg)
	{		
		switch(msg->what)
		{
			case	GROUP_1:
					if(g1->zGroupFolded()) g1->zUnfoldGroup();
					else g1->zFoldGroup();
					break;
			case	GROUP_2:
					if(g2->zGroupFolded()) g2->zUnfoldGroup();
					else g2->zFoldGroup();
					break;
			case	GROUP_3:
					if(g3->zGroupFolded()) g3->zUnfoldGroup();
					else g3->zFoldGroup();
					break;
		}
	};

	// Filter for messages which arrives to us.
	// Also determines if a ZGroup should be marked
	// or not within a drag&drop session.
	
	bool zMessageFilter(BMessage *msg)
	{
		// Not fine, but it works for this small demonstration
		return TRUE;
	};

};

// Message Handler for Bitmap Group
class Bitmap_Group : public ZGroup
{
	public:
	
	Bitmap_Group(TAG_ARG):ZGroup(TAG_SUPER) { }; 
	Bitmap_Group(struct TagItem *tags):ZGroup(tags) { };
	
	void zMessageReceived(BMessage *msg)
	{
		// We assume, that messages already got
		// through zMessageFilter()

		long state=(long)msg->FindLong(Z_DRAG_STATE);
		if(msg->Error()==B_NO_ERROR)
		{
			if(state==Z_DRAG_DROPPED)
			{
				ZGroup *src,*dst;
				if(src=(ZGroup *)msg->FindObject(Z_DRAG_SOURCE))
				{
					if(dst=(ZGroup *)msg->FindObject(Z_DRAG_DEST))
					{
						g1->zExchangeChilds(src,dst);
					}
				}
			}
		}
	};

	bool zMessageFilter(BMessage *msg)
	{
		if(msg->what=='brzh')	return TRUE;
		else					return FALSE;
	};
	
};

class ZTest:public BApplication
{
	public:
	
	ZTest():BApplication('ZTES')
	{
	};
	
	void AboutRequested()
	{
		BAlert *alert=new BAlert
		(
			"ZonicDemo",
			"ZonicDemo & Zonic V0.21 by Tinic Urou\n<5uro@informatik.uni-hamburg.de>",
			"I will leave him a line",
			NULL,
			NULL,
			B_WIDTH_FROM_LABEL
		);
		alert->Go();
	};
	
	void ReadyToRun()
	{
		ZGroup		*root;
		ZBTextView	*t1;
		ZBScrollBar *s1;
		ZWindow	*win;
		ZView		*view;


		// Initialise a new window		
		win = new ZWindow
		(
			// Zonic uses 'tags'
			// They are entered in this form:
			//
			// TagID, TagDATA
			//
			// TagDATA is of type int, but you can also pass pointers.
			
			ZWindow_Left,		100,
			ZWindow_Top,		100,
			ZWindow_Right,		400,
			ZWindow_Bottom,		440,
			ZWindow_Name, 		"Zonic V0.21, 25.Nov.1996",
			
			// Attach the view to the window
			ZWindow_View,	 	view = new ZView
			(
				ZView_Name,		"Zonic View",
				
				// Attach Groups to view
				ZView_Contents,	root = HGroupObject
				(
					ZGroup_Frame,			ZGroup_Frame_Root,
					ZGroup_Background, 		PEN_ROOT,
					Child, ZPagerObject
					(
						ZPager_Label, "Some nice classes",	
						Child, mg = new Button_Group
						(	
							ZGroup_Type, 			ZGroup_Type_Vertical,
							ZGroup_Frame,			ZGroup_Frame_None,
							ZGroup_MessageHandler, 	TRUE,
							Child,		  HGroupObject
							(
								Child, b1 = ZButtonObject 
								(
									ZButton_Label,	"_Bitmaps >>",
									ZButton_ShortCut, 'b',
									ZButton_MessageUp, new BMessage(GROUP_1),
									End
								),							
								Child, b2 = ZButtonObject 
								(
									ZButton_Label,	"_ZStringView >>",
									ZButton_ShortCut, 'z',
									ZButton_MessageUp, new BMessage(GROUP_2),
									End
								),							
								Child, b3 = ZButtonObject 
								(
									ZButton_Label,	"_Misc >>",
									ZButton_ShortCut, 'm',
									ZButton_MessageUp, new BMessage(GROUP_3),
									End
								),							
								End
							),			
							Child, g1 = new Bitmap_Group
							(
								ZGroup_Type,			ZGroup_Type_Horizontal,
								// Install a Message handler. An own thread will
								// be started for this.
								ZGroup_MessageHandler,	TRUE,
								ZGroup_Folded, 			TRUE,
								ZGroup_HWeight,			100,
								ZGroup_Label,	"Scaleable BBitmaps",
								Child, k1 = new ZBBitmap
								(	
									ZGroup_ID,			"breizh",
									ZGroup_CanBeDropped,TRUE,
									ZGroup_DragMessage,	new BMessage('brzh'), 
									ZGroup_Frame,		ZGroup_Frame_Standard,
									ZBBitmap_Right,		148,
									ZBBitmap_Bottom,	139,
									ZBBitmap_Depth,		B_COLOR_8_BIT,
									ZBBitmap_BitsDepth,	B_COLOR_8_BIT,
									ZBBitmap_Bits,		anImage,
									ZBBitmap_Length,	(152*140),
									ZBBitmap_FreeSize,	TRUE,
									End
								),					
								Child, k2 = new ZGroup
								(
									ZGroup_Type,		ZGroup_Type_Vertical,
									ZGroup_ID,			"breizh",
									ZGroup_CanBeDropped,TRUE,
									ZGroup_DragMessage,	new BMessage('brzh'), 
									ZGroup_Frame,		ZGroup_Frame_Standard,
									Child, ZGroupObject	(End),
									Child, ZButtonObject
									(
										ZButton_Label,	"Try to",
										End
									),
									Child, ZGroupObject	(End),
									Child, ZButtonObject
									(
										ZButton_Label,	"exchange",
										End
									),
									Child, ZGroupObject	(End),
									End
								),	
								Child, k3 = new ZBBitmap
								(
									ZGroup_ID,			"breizh",
									ZGroup_CanBeDropped,TRUE,
									ZGroup_DragMessage,	new BMessage('brzh'), 
									ZGroup_Frame,		ZGroup_Frame_Standard,
									ZBBitmap_Right,		138,
									ZBBitmap_Bottom,	122,
									ZBBitmap_Depth,		B_COLOR_8_BIT,
									ZBBitmap_BitsDepth,	B_COLOR_8_BIT,
									ZBBitmap_Bits,		BeImage,
									ZBBitmap_Length,	(140*123),
									ZBBitmap_FreeSize,	TRUE,
									End
								),
								Child, k4 = new ZGroup
								(
									ZGroup_Type,		ZGroup_Type_Vertical,
									ZGroup_ID,			"breizh",
									ZGroup_CanBeDropped,TRUE,
									ZGroup_DragMessage,	new BMessage('brzh'),
									ZGroup_Frame,		ZGroup_Frame_Standard,
									Child, ZGroupObject	(End),
									Child, ZButtonObject
									(
										ZButton_Label,	"these",
										End
									),
									Child, ZGroupObject (End),
									Child, ZButtonObject
									(
										ZButton_Label,	"groups",
										End
									),
									Child, ZGroupObject (End),
									End
								),
								End
							),				
							Child,  g2 = (ZBStringView *)ZBStringViewObject
							(
								ZGroup_Folded, TRUE,
								ZBStringView_Text,		"This is a ZBStringView",
								ZBStringView_Alignment,	B_ALIGN_CENTER,
								End
							),	
							Child, g3 = HGroupObject
							(
								ZGroup_Folded, TRUE,
								Child, ZPagerObject
								(
									ZGroup_HWeight,		100,
									ZGroup_DragMessage,	new BMessage('brzh'),
									
									ZPager_Label,	"Buttons",
									Child, VGroupObject
									(
										ZGroup_Frame,	ZGroup_Frame_None,
										Child, VGroupObject
										(	
											ZGroup_Label,	"ZBButton",
											Child, HGroupObject
											(
												Child, ZBButtonObject
												( 
													ZBButton_Label,	"Standard",
													End
												),
												Child, ZBButtonObject 	
												(
													ZBButton_Label,	"Interface",
													End
												),
												End	
											),
											Child, ZBButtonObject 
											(
												ZBButton_Label,	"Classes",
												End
											),
											End
										),		
										Child, VGroupObject
										(	
											ZGroup_Label,	"ZButton",
											Child, HGroupObject
											(
												Child, ZButtonObject
												( 
													ZButton_Label,	"_Custom",
													ZButton_ShortCut, 'c',
													End
												),
												Child, ZButtonObject 
												(
													ZButton_Label,	"Z_onic",
													ZButton_ShortCut, 'o',
													End
												),
												End	
											),
											Child, ZButtonObject 
											(
												ZButton_Label,	"Cl_asses",
												ZButton_ShortCut, 'a',
												End
											),
											End
										),
										End
									),					
									ZPager_Label,	"CheckBoxes",
									Child, HGroupObject
									(
										Space,
										Child, HGroupObject
										(							
											Child, VGroupObject
											(
												ZGroup_Align, ZGroup_Align_Left,
												Child, ZBCheckBoxObject 
												(
													ZBCheckBox_Label,	"CheckBoxes",
													End	
												),	
												Child, ZBCheckBoxObject
												(
													ZBCheckBox_Label,	"now look",
													ZBCheckBox_Value,	TRUE,
													End
												),
												Child, ZBCheckBoxObject 
												(
													ZBCheckBox_Label,	"okay",
													End
												),
												End
											),
											Child, VGroupObject
											(
												ZGroup_Align, ZGroup_Align_Left,
												Child, ZBCheckBoxObject
												( 
													ZBCheckBox_Label,	"but mine",
													ZBCheckBox_Value,	TRUE,
													End
												),
												Child, ZBCheckBoxObject
												(
													ZBCheckBox_Label,	"will",
													End
												),	
												Child, ZBCheckBoxObject 
												(
													ZBCheckBox_Label,	"be better",
													ZBCheckBox_Value,	TRUE,
													End
												),
												End
											),
											End
										),
										Space,	
										End					
									),
									ZPager_Label,	"RadioButtons",
									Child, HGroupObject
									(
										Space,
										Child, HGroupObject
										(							
											Child, ZBRadioAreaObject
											(
												Child, ZBRadioButtonObject 
												(
													ZBRadioButton_Label,	"RadioButton",
													End
												),	
												Child, ZBRadioButtonObject
												(
													ZBRadioButton_Label,	"look",
													End
												),
												Child, ZBRadioButtonObject 
												(
													ZBRadioButton_Label,	"okay",
													End
												),
												End
											),
											Child, ZBRadioAreaObject
											(
												Child, ZBRadioButtonObject
												( 
													ZBRadioButton_Label,	"but mine",
													End
												),
												Child, ZBRadioButtonObject
												(
													ZBRadioButton_Label,	"will be better",
													End
												),	
												End
											),
											End
										),
										Space,
										End							
									),
									ZPager_Label,	"StringEdit",
									Child, ZStringEditObject
									(
										ZStringEdit_Text,		"A simple ZStringEdit",
										End
									),
									End				
								),
								End
							),
							End
						),	
						ZPager_Label, "Read this!",	
						Child, HGroupObject
						(
							ZGroup_Space,	0,	
							Child, t1 = ZBTextViewObject
							(
								ZGroup_HWeight,		70,
								ZBTextView_WordWrap,	TRUE,
								ZBTextView_Editable,	TRUE,
								End
							),	
							Child, s1 = ZBScrollBarObject
							(
								ZBScrollBar_Orientation,B_VERTICAL,
								End
							),
							End
						),
						End
					),
					End	
				),
				End
			),
			End
		);
		
		// Setup target message handler for groupobjects
		// Naturally this cant be done with the constructortags.
		
		b1->zSetTarget(mg);
		b2->zSetTarget(mg);
		b3->zSetTarget(mg);
		
		k1->zSetTarget(g1);
		k2->zSetTarget(g1);
		k3->zSetTarget(g1);
		k4->zSetTarget(g1);
		
		s1->SetTarget(t1);
		
		t1->SetText
		(	
			"This simple demo shows, that an autolayout engine "
			"is possible on the BeBox. The creation of GUIs "
			"is very easy, since no fixed coordinates are used."
			"\n\nTry resizing this window."
			"\n\nIf you have questions or want to contribute "
			"to this project, write to:\n"
			"\n\t5uro@informatik.uni-hamburg.de "
			"\n\nThis project is supported by the DeBUG Usergroup. "
			"Visit the DeBUG center at: "
			"\n\nhttp://www.ai-lab.fh-furtwangen.de/~DeBUG "
			"\n\n 1996 by Tinic Urou"
		);
					
		win->Show();
	};
};

int main()
{
	ZTest *my_app=new ZTest();
	my_app->Run();
	delete my_app;	
	return 0;
}